<?php
namespace CashBill\Payments\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Store\Model\ScopeInterface;
use Magento\Sales\Model\Order;
use CashBill\Payments\Model\Library\Shop;

final class Notify extends AbstractHelper
{   
    const POSITIVE_FINISH = 'PositiveFinish';
    const ABORT = 'Abort';
    const NEGATIVE_AUTHORIZATION = 'NegativeAuthorization';
    
    public function process(array $data)
    {
        $signature = md5($data['cmd'] . $data['args'] . $this->getConfigData('cb_secret'));
        
        if($data['sign'] !== $signature){
            exit("Signature error");
        }
        
        $shop = new Shop($this->getConfigData('cb_id'), $this->getConfigData('cb_secret'), $this->getConfigData('sandbox') != true);
        $transactionInfo = $shop->getFullPaymentInfo($data['args']);
        if(!in_array($transactionInfo->status , array(self::POSITIVE_FINISH, self::ABORT, self::NEGATIVE_AUTHORIZATION ))){
            exit("OK");
        }
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $orderFactory = $objectManager->get('Magento\Sales\Model\OrderFactory');
        $order        = $orderFactory->create()->loadByAttribute('entity_id', $transactionInfo->additionalData);
        if(intval($order->getGrandTotal() * 100) !== intval($transactionInfo->amount->value * 100)){
            exit("wrong amount");
        }

        if($order->getStatus() === Order::STATE_COMPLETE){
            exit("OK");
        }

        switch($transactionInfo->status){
            case self::POSITIVE_FINISH :
            $this->complete($order,$transactionInfo);
            break;
            case self::ABORT:
            case self::NEGATIVE_AUTHORIZATION:
            $this->reject($order);
            break;
        }

        exit("OK");
    }

    private function complete($order,$transactionInfo)
    {
        $payment = $order->getPayment();
        $payment->setTransactionId($transactionInfo->id);
        $payment
            ->setIsTransactionApproved(true)
            ->setShouldCloseParentTransaction(true)
            ->setIsTransactionClosed(true);

        if ($order->canInvoice()) {
            $invoice = $order->prepareInvoice();
            $invoice->register();
            $invoice->setTransactionId($transactionInfo->id);
            $order->addRelatedObject($invoice);
        }
        $order->addStatusHistoryComment('Płatność została zaksięgowana w systemie CashBill.', Order::STATE_COMPLETE);
        $order->getResource()->save($order);
    }

    private function reject($order,$transactionInfo)
    {
        $order->cancel();
        $order->addStatusHistoryComment('Płatność została anulowana przez klienta.');
        $order->getResource()->save($order);
    }

    public function getConfigData($field)
    {
        return $this->scopeConfig->getValue('payment/cashbill_payments/' . $field,
            ScopeInterface::SCOPE_STORE);
    }
}
